/* NIGHTFALL Light Curve Synthesis Program                                 */
/* Copyright (C) 1998 Rainer Wichmann                                      */
/*                                                                         */
/*  This program is free software; you can redistribute it                 */
/*  and/or modify                                                          */
/*  it under the terms of the GNU General Public License as                */
/*  published by                                                           */
/*  the Free Software Foundation; either version 2 of the License, or      */
/*  (at your option) any later version.                                    */
/*                                                                         */
/*  This program is distributed in the hope that it will be useful,        */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of         */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          */
/*  GNU General Public License for more details.                           */
/*                                                                         */
/*  You should have received a copy of the GNU General Public License      */
/*  along with this program; if not, write to the Free Software            */
/*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.              */

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <float.h>
#include "Light.h"



/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Attach the throat of star a to star b
 @tip       This is to verify eclipse by 'own' star
 @param     (void) 
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/
void LightCopyThroat()
{
  long   j;                                 /* loop counter                 */
  long  Nplus = 0;                          /* surface elements + throat    */
  long  NP = 0;                             /* surface elements primary     */
  long NS = 0;                              /* surface elements secondary   */

  /* --------  attach Secondary throat to Primary ----------------------    */

  Nplus = Binary[Secondary].N_PhiStep[StepsPerPi-1]
    + Binary[Secondary].N_PhiStep[StepsPerPi-2];
  NP    = Binary[Primary].NumElem;
  NS    = Binary[Secondary].NumElem;


  Binary[Primary].NumPlus = NP + Nplus;
  if (Binary[Primary].NumPlus > (long) maxElements) 
    nf_error(_(errmsg[10]));

  for (j = 0; j < Nplus; ++j) {

    Surface[Primary][NP+j] 
      = Surface[Secondary][NS-Nplus+j];

    /* ---------------  INVERT X ---------------------------------------   */

    Surface[Primary][NP+j].lambda = (float)
      (1.0 - Surface[Primary][NP+j].lambda);

    Surface[Primary][NP+j].l =
      (-Surface[Primary][NP+j].l);

  }

  /* ---------------- attach Primary throat to Secondary ----------------  */

  Nplus = Binary[Primary].N_PhiStep[StepsPerPi-1]
    + Binary[Primary].N_PhiStep[StepsPerPi-2];

  Binary[Secondary].NumPlus = NS + Nplus;

  if (Binary[Primary].NumPlus > (long) maxElements) 
    nf_error(_(errmsg[10]));

  for (j = 0; j < Nplus; ++j) {

    Surface[Secondary][NS+j] 
      = Surface[Primary][NP-Nplus+j]; 

    /* ---------------------  INVERT X  ---------------------------------  */

    Surface[Secondary][NS+j].lambda = (float)
      (1.0 - Surface[Secondary][NS+j].lambda);

    Surface[Secondary][NS+j].l =
      (-Surface[Secondary][NS+j].l);

  }

  return;
}                            

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Copy the throat back
 @param     (void) 
 @return    (void) 
 @heading   Light Curve
 ****************************************************************************/
void LightCopyThroatBack()
{
  long   j;                                  /* loop counter                 */
  long  Nplus = 0;                           /* surface elements + throat    */
  long  NP = 0;                              /* surface elements primary     */
  long  NS = 0;                              /* surface elements secondary   */

  NP    = Binary[Primary].NumElem;
  NS    = Binary[Secondary].NumElem;

  /* -----------  copy back Primary throat to Primary -------------------   */

  Nplus = Binary[Secondary].NumPlus - NS;

  for (j = 0; j < Nplus; ++j) {

    if(Surface[Secondary][NS+j].EclipseFlag == ON){

      Surface[Primary][NP-Nplus+j].EclipseFlag 
	= Surface[Secondary][NS+j].EclipseFlag;

      Surface[Primary][NP-Nplus+j].visibility 
	= Surface[Secondary][NS+j].visibility;

    }
    
  }

  /* -------------  copy back Secondary throat to Secondary -------------   */

  Nplus = Binary[Primary].NumPlus - NP;
  
  for (j = 0; j < Nplus; ++j) {

    if (Surface[Primary][NP+j].EclipseFlag == ON){

      Surface[Secondary][NS-Nplus+j].EclipseFlag 
	= Surface[Primary][NP+j].EclipseFlag;

      Surface[Secondary][NS-Nplus+j].visibility 
	= Surface[Primary][NP+j].visibility;

    }
  }

  return;
}

/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Calculate various quantities for simple geometric tests
 @param     (void) 
 @return    (int)  Exit status 
 @heading   Light Curve
 ********************************************************************/
int LightSetupTests()
{

  int      testerr = 0;     /* Exit status RootFind                 */

  /* double   Xl, h1, h2; */      /* Xl lagrange one                      */
  double   Xp1, Xp2;        /* max. distance surface-center of star */
  /* double   PhiLagrange; */     /* tangent angle of roche lobes at L1   */
  double   CosPhiLagrange;  /* tangent angle at Lagrange One (cos)  */
  double   CosPhiOne = 0.0; /* perhaps more stringent if Roche lobe */
                            /* not filled                           */

  /* ----------  simple geometric tests for eclipse possible ------ */ 


  /* osculating cone angle at L1                                    */
  /*
  Xl    = Binary[Primary].RLag1;
  h1    = 1.0/(Xl*Xl*Xl);
  h2    = 1.0/((1.0-Xl)*(1.0-Xl)*(1.0-Xl));
  PhiLagrange = sqrt((2*h1 + 2*Mq*h2 + (1.0 - Mq))/(h1 + Mq*h2 - (1.0 - Mq)));
  PhiLagrange = atan(PhiLagrange);
  CosPhiLagrange = cos(PhiLagrange);
  */

  Xp1 = 1.0; Xp2 = 1.0;

  /* calculate Xp == distance from center to surface towards L1     */

  /* lambda, nu global; potential on x-axis                         */

  lambda     = 1.0; nu = 0.0;  
  RochePot   = Binary[Primary].RochePot;
  Mq         = Binary[Primary].Mq;
  if (Flags.asynchron1 == ON)
    F        = Binary[Primary].Fratio;
  else
    F        = 1.0;


  if (fabs(Binary[Primary].RocheFill - 1.0) <= FLT_EPSILON) { 
    Xp1     = Binary[Primary].RXCrit;
  } else {
    if (Flags.fill == OFF) {
      Xp1     = RootFind(RocheSurface, 0.00001, Binary[Primary].RXCrit, 1.0e-8,
			 "LightSetupTests", &testerr);
      if (testerr == 1) return(8);
    } else {
      Xp1     = Binary[Primary].LimRad;
    }
  }
  Binary[Primary].Xp = Xp1;

  RochePot   = Binary[Secondary].RochePot;
  Mq         = Binary[Secondary].Mq;
  if (Flags.asynchron2 == ON)
    F        = Binary[Secondary].Fratio;
  else
    F        = 1.0;


  if (fabs(Binary[Secondary].RocheFill - 1.0) <= FLT_EPSILON) {
    Xp2     = Binary[Secondary].RXCrit;
  } else {
    if (Flags.fill == OFF) {
      Xp2     = RootFind(RocheSurface, 0.00001,Binary[Secondary].RXCrit, 
			 1.0e-8,
			 "LightSetupTests", &testerr );
      if (testerr == 8) return(8);
      
    } else {
      Xp2     = Binary[Secondary].LimRad;
    }
  }
  Binary[Secondary].Xp = Xp2;

  if (Flags.debug[GEO] == ON)
    {
      printf("\nXp[P]: %6.2f  Xp[S]: %6.2f\n", 
	     Binary[Primary].Xp, Binary[Secondary].Xp);
    }

  /* ------  sanity check if F < 1.0 ------------------------------ */

  if (
      ( (Binary[Secondary].Fratio - 1.0) <= (-FLT_EPSILON)
	&& Flags.asynchron1 == ON )
      || ( (Binary[Primary].Fratio - 1.0) <= (-FLT_EPSILON) 
	   && Flags.asynchron2 == ON ) ) {
    if ((Binary[Secondary].Xp + Binary[Primary].Xp) >= (1.0+DBL_EPSILON) ) {
      WARNING(_("Fill Factor too large in Async Rotating Star --> Stars intersect"));
      return(1);
    }
  }

  if (Flags.fill == OFF) {
    CosPhiOne  = sqrt( MAX(0.0, (1.0 - SQR(Xp1 + Xp2))) );
  } else {
    CosPhiOne  = 0.1;
  }

  /* osculating cone angle at L1                                    */

  /* Chanan et al. (1972), ApJ 208, 512
   */

  if ( MIN(Binary[Primary].Mq, Binary[Secondary].Mq) > 0.4)
    CosPhiLagrange = cos (DTOR*57.88);
  else if ( MIN(Binary[Primary].Mq, Binary[Secondary].Mq) > 0.2)
    CosPhiLagrange = cos (DTOR*59.00);
  else if ( MIN(Binary[Primary].Mq, Binary[Secondary].Mq) > 0.1)
    CosPhiLagrange = cos (DTOR*60.56);
  else if ( MIN(Binary[Primary].Mq, Binary[Secondary].Mq) > 0.01)
    CosPhiLagrange = cos (DTOR*67.17);
  else 
    CosPhiLagrange = 0.0;
  
  /* exact contact system
   */
  if ( (fabs(Binary[Primary].RocheFill - 1.0) <= FLT_EPSILON) &&
       (fabs(Binary[Secondary].RocheFill - 1.0) <= FLT_EPSILON) ) { 
    CosPhiLagrange = CosPhiLagrange;
  } else {
    CosPhiLagrange = CosPhiOne;
  }

  Binary[Primary].CosPhiLagrange   = CosPhiLagrange;
  Binary[Secondary].CosPhiLagrange = CosPhiLagrange;
#ifdef HAVE_DISK
  /*
   * CosPhiLagrange for Disk; same formula as for CosPhiOne, but
   * now with the outer disk radius.
   */
  Xp2 = Binary[Disk].Rout * Binary[Secondary].RLag1;
  Binary[Disk].CosPhiLagrange = sqrt( MAX(0.0, (1.0 - SQR(Xp1 + Xp2))) );
#endif
   
  return(0);
}
           
#ifdef HAVE_DISK

void SecondaryEclipsedByDisk (int j, double CosPhase2, double SinPhase2, 
			      double lzero, double mzero, double nzero);
void PrimaryEclipsedByDisk   (double CosPhase,  double SinPhase, double Phase,
			      double lzero, double mzero);
void DiskEclipsedByDisk      (double CosPhase2, double SinPhase2, 
			      double lzero, double mzero);


/********************************************************************
 @package   nightfall
 @author    Patrick Risse (risse@astro.uni-tuebingen.de)
 @version   1.0
 @short     Eclipse Verification for a CB-System with a Disk
 @param     (int)  j The phase index 
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void LightCurveDisk(int j)
{

  long NElemD;                           /* # of surface elements   */ 
  long i;                                /* loop counter            */
  int  eclipseS=0;
  int  eclipseP=0; 
  int  eclipseD=0; 
  int  count=0;
  int  invis=0;
  int  intersect = 0;
  int  testsec = 0;
  int  tested = 0;
  int  EclipseImpossible = 0;

  int       eclipsed = 0;                /* test variable           */

  const int No  = 0;                     /* test value              */
  const int Invisible = 10;              /* test value              */

  int      MinErr;                       /* exit status MinFinder   */
  double   Phase, Phase2;                /* orbital phase           */

  double   lzero,  mzero,  nzero;        /* LOS vector              */
  double   lzero2, mzero2, nzero2;       /* LOS vector              */
  double   x_z, y_z, z_z;          
  double   x_tl, y_tl, z_tl;
  double   Qi, Li, QLi;                  /* for sphere test         */   
  double   t_l, t_1, t_2, t_dist;        /* intersection w/ sphere  */

  double   tmin = 0;        /* minimum along LOS                    */
  double   PotMin;          /* minimum along LOS                    */
  double   SqrXp1;          /* square of (center - lagrange one)    */
  double   SqrPol1,SqrPol2; /* square of polar radius               */
  double   Sini, Cosi;      /* cos, sin of orbital inclination      */
  double   SinPhase, CosPhase; /* cos, sin of phase (primary)       */
  double   SinPhase2, CosPhase2; /* cos, sin of phase (secondary)   */
  double   CosPhiLagrange;  /* tangent angle at Lagrange One (cos)  */

  double   Xp1, Xp2;        /* max. distance surface-center of star */
  double   Pot_One, Pot_Two; /* surface potentials                  */

  SurfaceElement *SurfPtrD;  /* pointer to surface Disk             */

  double   fratio1 = 1.0;    /* async rotation                      */
  double   fratio2 = 1.0;    /* async rotation                      */

  double   OmegaDot;         /* velocity calculation                */
  double   SecondaryMass;    /* velocity calculation                */
  double   SemiAxis;         /* velocity calculation                */
  double   KeplerPeriod;     /* velocity calculation                */
  double   G_SI = 6.6726e-11;/* gravitational constant              */

  /* COMMENTS IN ENGLISH PLEASE !!!!!!!!! MK                        */
  /*double vec_e1[3];*/          /* Ebenenvektor 1                      */
  /*double vec_e2[3];*/          /* Ebenenvektor 2                      */
  /*double vec_p1[3];*/          /* Eckpunkt der Ebene                  */
  /*double vec_en[3];*/          /* Normalenvektor der Ebene            */
  /*double vec_g1[3];*/          /* Geradenvektor                       */
  /*double vec_p2[3];*/          /* Eckpunkt der Gerade                 */
  /*double vec_dif_p2p1[3];*/    /* Differenze der beiten Ortsvektoren  */
  /*double det1,det2;*/          /* Determinanten                       */
  /* double vec_intersection[3];*/ /* Schnittpunkt zwischen Gerade und Ebene */
  /* double Distance[3]; */   /* Distanz zwischen Schnittpunkt und 
				Flaechenmittelpunkt                 */
  /* double r;   */           /* Faktor                              */
  /* double rad_angle,angle; */   /* Angle between two vectors           */

  /* Recalculate the position of the Disk */
  /* DivideDisk(Disk,j); */

  if (Flags.asynchron1 == ON) fratio1 = Binary[Primary].Fratio;
  if (Flags.asynchron2 == ON) fratio2 = Binary[Secondary].Fratio;


  Sini  = sin(Orbit.Inclination);
  if (fabs(Sini) <= DBL_EPSILON) 
    Sini = 0.0;
  Cosi  = cos(Orbit.Inclination);
  if (fabs(Cosi) <= DBL_EPSILON) 
    Cosi = 0.0; 


  Mq    = Binary[Primary].Mq;       /* Mq is global variable        */ 

  Pot_One        = Binary[Primary].RochePot;
  Pot_Two        = Binary[Secondary].RochePot;
  Xp1            = Binary[Primary].Xp;
  Xp2            = Binary[Secondary].Xp;

  CosPhiLagrange = Binary[Disk].CosPhiLagrange; 

  SqrXp1  = SQR(Xp1);

  SqrPol1 = SQR(Binary[Primary].Radius);  /* square of polar radius */
  SqrPol2 = SQR(Binary[Secondary].Radius);

  /* ------------------     eclipse test         ------------------ */

  /* to allow for generalization, we dont insist on                 */
  /*  same number of surface elements on both components            */


  NElemD = Binary[Disk].NumElem;

  /* We start at 90 deg, Phase = -0.25, both stars fully visible    */

  if (Flags.elliptic == OFF) {

    Phase = (PI/2 + (2*PI/PhaseSteps) * j ); 
    FluxOut[j].Phase = (float) Phase;
    Orbit.Phase = Phase;
    Orbit.Nu    = Phase;

  } else {
    
     Phase = Orbit.Phase;
     /* Orbit.OmegaZero is mean anomaly at secondary eclipse        */
     FluxOut[j].Phase = (float) (Orbit.M + PI + Orbit.OmegaZero);
  }

  SinPhase = sin(Phase); 
  if (fabs(SinPhase) <= DBL_EPSILON) 
    SinPhase = 0.0;
  CosPhase = cos(Phase);
  if (fabs(CosPhase) <= DBL_EPSILON) 
    CosPhase = 0.0;

  /* vector towards observer (LOS = Line Of Sight)                  */
  lzero   =  Sini * CosPhase;
  mzero   =  Sini * SinPhase;
  nzero   =  Cosi;

  /* Secondary turn by PI and mirror y->(-y)                        */
  /* this is equivalent to mirror by x->(-x)                        */

  Phase2 = (PI + Phase);
  SinPhase2 = sin(Phase2); 
  if (fabs(SinPhase2) <= DBL_EPSILON) 
    SinPhase2 = 0.0;
  CosPhase2 = cos(Phase2);
  if (fabs(CosPhase2) <= DBL_EPSILON) 
    CosPhase2 = 0.0;

  /* vector towards observer (LOS = Line Of Sight)                  */
  lzero2   =  Sini * CosPhase2;
  mzero2   =  Sini * SinPhase2;
  nzero2   =  Cosi;
 
  SurfPtrD = Surface[Disk];

  
  /* >>>>>>>>>>>>>>>>>  the disk  <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< */

  /******       NEW CODE by MPR 29.04.2002     **********************/
  
  /* Disk surrounds Secondary. Non-tilted display in Gnuplot ok.    */

  /* loop over surface elements                                     */

  for (i = 0; i < NElemD; ++i) 
    {
#if 0
      if (fabs(SurfPtrD->lambda) <= DBL_EPSILON)
	SurfPtrD->lambda = 0.0;
      if (fabs(SurfPtrD->mu)     <= DBL_EPSILON)
	SurfPtrD->mu     = 0.0;
      if (fabs(SurfPtrD->nu)     <= DBL_EPSILON)
	SurfPtrD->nu     = 0.0;   
#endif

      /* Need Kepler orbital period for disc element  */

      SecondaryMass = Orbit.TrueMass * Binary[Primary].Mq / ( 1.0 + Binary[Primary].Mq);
      /* distance from secondary                                    */
      SemiAxis      = Orbit.TrueDistance * SurfPtrD->rad;

      /* orbit around secondary                                     */
      KeplerPeriod  = 2.0 * PI *
	sqrt((SemiAxis * SemiAxis * SemiAxis) / (G_SI * SecondaryMass));
      
      /* OmegaDot is rotation velocity in rad/s                     */
      OmegaDot           = 2.0 * PI / KeplerPeriod;
	  
      SurfPtrD->Velocity = 
	- OmegaDot          * SemiAxis
	* (SurfPtrD->mu     * lzero2 + SurfPtrD->lambda * mzero2);

      /* gamma = angle surface normal - LOS                         */
      /*  dont forget to mirror in y->(-y)                          */

      SurfPtrD->CosGamma = ( lzero2   * SurfPtrD->l
			     + mzero2 * (-SurfPtrD->m)
			     + nzero2 * SurfPtrD->n );

      /*
      if (i == 0) {
	printf ("lzero2: %5.3f  mzero2:  %5.3f   nzero2:  %5.3f  CosGamma: %5.3f\n", 
		lzero2, mzero2, nzero2,
		SurfPtrD->CosGamma);
      }
      */

      /*=========================================================== */

      /* Default is visible                                         */

      eclipsed = No; 

      /* Start checking.                                            */ 
	  
      /* 1. Does the surface normal point away from observer ? 
       *    (Tested, works.)
       */
      if (SurfPtrD->CosGamma < DBL_EPSILON )
	{
	  eclipsed = Invisible;
	} 

      if (eclipsed == No)
	{
	  /* 2. Is element eclipsed by secondary ?
	   *    (Tested, works for non-inclined disk.)
	   *
	   *    Test for an intersection with surface of Secondary.
	   *    Both Secondary and disk are at (0,0,0). Phase is shifted 
	   *    by PI, and y->(-y).
	   */
	  x_z = SurfPtrD->lambda;
	  y_z = (-SurfPtrD->mu); 
	  z_z = SurfPtrD->nu;
	  Qi  = 2.0 * ( x_z*lzero2 + y_z*mzero2 + z_z*nzero2 );
	  Li  = x_z*x_z + y_z*y_z + z_z*z_z - (Xp2)*(Xp2);
	  QLi = Qi*Qi - 4.0*Li;


	  /* If true there is an intersection with the 
	   * sphere of the secondary star. Further tests 
	   * are required 
	   */
	  if (QLi <= 0.0)
	    {
	      /* no intersection */;
	    }
	  else
	    {
 
	      t_l  = sqrt(QLi);
	      t_1  = (-Qi+t_l)/2;
	      t_2  = (-Qi-t_l)/2;

	      if  (t_1 > 0.0 && t_2 > 0.0)
		{
		  /* check whether distance is greater than polar radius */
		  /*  -- maximum inscribed sphere                        */
		  t_l = (t_1 + t_2)/2.0; 
		  
		  /* t_l is Midpoint of ray intersecting Xp2-Sphere      */
		  x_tl = x_z + t_l * lzero2;
		  y_tl = y_z + t_l * mzero2;
		  z_tl = z_z + t_l * nzero2;
		  t_dist = SQR(x_tl) + y_tl * y_tl + z_tl * z_tl;
		  
		  if (t_dist <= (SqrPol2-DBL_EPSILON)) 
		    { 
		      /* definitely eclipsed  
		       */
		      eclipsed = EclipsedBySecondary;  
		    } 
		  else 
		    {
		      
		      /* Find Potential Minimum between t_1 and t_2       
		       * final resort                                     
		       * changed Binary[Pri].Mq to Binary[Sec].Mq Jul 6   
		       */
		      MinErr = MinFinder(&Binary[Secondary].Mq, 
					 &fratio2, 
					 &t_1, &t_2, 
					 &lzero, &mzero, &nzero,
					 &x_z, &y_z, &z_z,
					 &tmin, &PotMin);
		      if (MinErr == 1 && Flags.fill == OFF) 
			WARNING(_("LightCurve: No Minimum Found"));
		      PotMin = -PotMin; 
		      SurfPtrD->LOS_Pot = (float) PotMin;
		      
		      /* added condition tmin >= 0          
		       */
		      if (PotMin >= Pot_Two && tmin >= 0) 
			{ 
			  eclipsed = EclipsedBySecondary;
			}

		    }
		} /* if  (t_1 > 0.0 && t_2 > 0.0) */
	    } /* QLi > 0.0 */
	}
	

      if (eclipsed == No && CosPhase2 > (CosPhiLagrange-DBL_EPSILON))
	{ 
	  ++testsec;

	  if (eclipsed == No)
	    {
	      /* 3. Is the element eclipsed by the primary?
	       *    (Tested, works for non-inclined disk.)
	       */
	      x_z = SurfPtrD->lambda;
	      y_z = (-SurfPtrD->mu);
	      z_z = SurfPtrD->nu;

	      Qi  = 2.0*( x_z*lzero2 + y_z*mzero2 + z_z*nzero2 - lzero2);
	      Li  = 1.0 + x_z*x_z + y_z*y_z + z_z*z_z - 2*x_z - SqrXp1;
	      QLi = Qi*Qi - 4.0*Li;

	      if (QLi <= 0.0)
		{
		  /* no intersection */;
		}
	      else 
		{
		  t_l = sqrt(QLi);    /* just to hold temporary result   */
		  t_1 = (-Qi + t_l)/2.0;
		  t_2 = (-Qi - t_l)/2.0;

		  if  (t_1 > 0.0 && t_2 > 0.0)  /* intersection */
		    {
		      ++intersect;
		      
		      /* check whether distance is greater than polar radius */
		      /* -- maximum inscribed sphere                         */
		      t_l = sqrt(QLi);    /* just to hold temporary result   */
		      t_1 = (-Qi + t_l)/2.0;
		      t_2 = (-Qi - t_l)/2.0;
		      t_l = (t_1 + t_2)/2.0;
		      
		      /* t_l is Midpoint of ray intersecting Xp1-Sphere      */
		      x_tl = x_z + t_l * lzero2;
		      y_tl = y_z + t_l * mzero2;
		      z_tl = z_z + t_l * nzero2;
		      t_dist = SQR(x_tl-1.0) + y_tl * y_tl + z_tl * z_tl;
		      
		      /* we do not need the sqrt, compare the squares        */
		      if (t_dist <= (SqrPol1-DBL_EPSILON)) 
			{ 
			  /* definitely eclipsed 
			   */
			  eclipsed = EclipsedByPrimary ;
			} 
		      else 
			{
			  ++tested;
			  /* Find Potential Minimum between t_1 and t_2      */
			  /* final resort                                    */
			  /* changed Binary[Pri].Mq to Binary[Sec].Mq Jul 6  */
			  MinErr = MinFinder(&Binary[Primary].Mq, 
					     &fratio1, 
					     &t_1, &t_2, 
					     &lzero2, &mzero2, &nzero2,
					     &x_z, &y_z, &z_z,
					     &tmin, &PotMin);
			  if (MinErr == 1) 
			    WARNING(_("LightCurve: No Minimum Found"));
			  PotMin = -PotMin; 
			  SurfPtrD->LOS_Pot = (float) PotMin;
			  
			  /* added condition tmin >= 0                       */
			  if (PotMin >= Pot_One && tmin >= 0) 
			    { 
			      eclipsed = EclipsedByPrimary; 
			    }
		      
			} /* t_dist > (SqrPol1-DBL_EPSILON */
		    } /* if  (t_1 > 0.0 && t_2 > 0.0) */
		} /* QLi > 0.0) */
	    } /* (eclipsed < 1 )*/
	}
      else
	{
	  ++EclipseImpossible;
	}


      if (eclipsed == EclipsedByPrimary)	    
	eclipseP = eclipseP+1;
      if (eclipsed == EclipsedByDisk)
	eclipseD = eclipseD+1;
      if (eclipsed == EclipsedBySecondary)
	eclipseS = eclipseS+1;
      if (eclipsed == Invisible)
	invis = invis+1;
      if (eclipsed == No)
	{
	  count=count+1;
	}


      SurfPtrD->EclipseFlag = eclipsed;
      
      if (SurfPtrD->EclipseFlag > 0 ) SurfPtrD->visibility = 0;
      else SurfPtrD->visibility = 1;
      if (eclipsed == 10) SurfPtrD->CosGamma = -1;
      ++SurfPtrD;

    } /* end loop over surface elements */

  if (Flags.debug[GEO] && Flags.debug[VERBOSE])
    { /* at least things look okay here */
      printf("=========================================\n");
      printf("Step   %02d\n", j);
      printf("Total  %d  visible %d  invisible %d  eclipsed_tot %d\n",
	     count+invis+eclipseP+eclipseS+eclipseD,
	     count,invis,eclipseP+eclipseS+eclipseD);
      printf("eclipsed: by Prim %d, by Sec %d, by Disk %d\n",
	     eclipseP,eclipseS,eclipseD);
      printf("testsec  : %d\n", testsec);
      printf("intersect: %d\n", intersect);
      printf("tested   : %d\n", tested);
      printf("imposs.  : %d\n", EclipseImpossible);
      /* check all four disk areas */
      int c_v[4] = { 0 };
      int c_i[4] = { 0 };
      int c_p[4] = { 0 };
      int c_s[4] = { 0 };
      int c_d[4] = { 0 };
      
      SurfPtrD = Surface[Disk];
      for (i = 0; i < NElemD; ++i) 
	{
	  int index = SurfPtrD->AreaType - 1;
	  if (SurfPtrD->EclipseFlag == EclipsedByPrimary)
	    ++c_p[index];
	  else if (SurfPtrD->EclipseFlag == EclipsedBySecondary)
	    ++c_s[index];
	  else if (SurfPtrD->EclipseFlag == EclipsedByDisk)
	    ++c_d[index];
	  else if (SurfPtrD->EclipseFlag == Invisible)
	    ++c_i[index];
	  else
	    ++c_v[index];
	  ++SurfPtrD;
	}
      printf("TOP   : by Prim %6d, by Sec %6d, by Disk %6d, invisible: %6d, visible %6d\n",
	     c_p[0], c_s[0], c_d[0], c_i[0], c_v[0] );
      printf("OUT   : by Prim %6d, by Sec %6d, by Disk %6d, invisible: %6d, visible %6d\n",
	     c_p[1], c_s[1], c_d[1], c_i[1], c_v[1] );
      printf("BOTTOM: by Prim %6d, by Sec %6d, by Disk %6d, invisible: %6d, visible %6d\n",
	     c_p[2], c_s[2], c_d[2], c_i[2], c_v[2] );
      printf("IN    : by Prim %6d, by Sec %6d, by Disk %6d, invisible: %6d, visible %6d\n",
	     c_p[3], c_s[3], c_d[3], c_i[3], c_v[3] );
      
    }

  count=0;
  eclipseP=0;
  eclipseS=0;
  invis=0;
  intersect=0;

  SecondaryEclipsedByDisk (j, CosPhase2, SinPhase2, lzero2, mzero2, nzero2);
  DiskEclipsedByDisk      (CosPhase2, SinPhase2, lzero2, mzero2);

  if (-CosPhase2 > (CosPhiLagrange-DBL_EPSILON))
    {
      /* checked, correct condition */
      PrimaryEclipsedByDisk (CosPhase, SinPhase, Phase, lzero, mzero);
    }

  return;

} /* end LightCurveDisk*/
#endif

/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Eclipse Verification
 @param     (int)  j The phase index 
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void LightCurve(int j)
{
  const int  primary_eclipsed   = 1;
  const int  secondary_eclipsed = 2;
  const int  primary_total      = 4;
  const int  secondary_total    = 8;

  long       np_eclipsed = 0;
  long       ns_eclipsed = 0;
  long       np_visible  = 0;
  long       ns_visible  = 0;

  static int contactflag = 0;            /* changes at contacts     */
  int        contactflag_old;            /* to check for changes    */

  long NElem, NElemP, NElemS;            /* # of surface elements   */ 
  long i;                                /* loop counter            */
  int       eclipsed = 1;                /* test variable           */
  const int Yes = 1;                     /* test value              */
  const int No  = 0;                     /* test value              */
  const int Invisible = 10;              /* test value              */
  int       MinErr;                      /* exit status MinFinder   */

  double   Phase, Phase2,M_Phase;        /* orbital phase           */
  double   OmegaDotPrimary;              /* surface rotation async  */
  double   OmegaDotSecondary;            /* surface rotation async  */
  double   lzero,  mzero,  nzero;        /* LOS vector              */
  double   lzero2, mzero2, nzero2;       /* LOS vector              */

  double   x_z, y_z, z_z;          
  double   x_tl, y_tl, z_tl;
  double   Qi, Li, QLi;                  /* for sphere test         */   
  double   t_l, t_1, t_2, t_dist;        /* intersection w/ sphere  */

  double   tmin = 0;        /* minimum along LOS                    */
  double   PotMin;          /* minimum along LOS                    */
  double   SqrXp1, SqrXp2;  /* square of (center - lagrange one)    */
  double   SqrPol1,SqrPol2; /* square of polar radius               */
  double   Sini, Cosi;      /* cos, sin of orbital inclination      */
  double   SinPhase, CosPhase; /* cos, sin of phase (primary)       */
  double   SinPhase2, CosPhase2; /* cos, sin of phase (secondary)   */
  double   Cosgamma;        /* angle between LOS and surface normal */
  double   CosPhiLagrange;  /* tangent angle at Lagrange One (cos)  */

  double   Xp1, Xp2;        /* max. distance surface-center of star */
  double   Pot_One, Pot_Two; /* surface potentials                  */

  SurfaceElement *SurfPtrP;  /* pointer to surface Primary          */
  SurfaceElement *SurfPtrS;  /* pointer to surface Secondary        */

  double   fratio1 = 1.0;    /* async rotation                      */
  double   fratio2 = 1.0;    /* async rotation                      */

  if (Flags.asynchron1 == ON) fratio1 = Binary[Primary].Fratio;
  if (Flags.asynchron2 == ON) fratio2 = Binary[Secondary].Fratio;

  OmegaDotPrimary   = fratio1   * 2.0 * PI / Orbit.TruePeriod;
  OmegaDotSecondary = fratio2   * 2.0 * PI / Orbit.TruePeriod;

  Sini  = sin(Orbit.Inclination);
  Cosi  = cos(Orbit.Inclination);
  NElem = Binary[Primary].NumElem; 
  Mq    = Binary[Primary].Mq;       /* Mq is global variable        */ 

  Pot_One        = Binary[Primary].RochePot;
  Pot_Two        = Binary[Secondary].RochePot;
  Xp1            = Binary[Primary].Xp;
  Xp2            = Binary[Secondary].Xp;

  /* CosPhiLagrange is same for Primary and Secondary               */
  CosPhiLagrange = Binary[Primary].CosPhiLagrange; 

  SqrXp1 = SQR(Xp1);
  SqrXp2 = SQR(Xp2);
  SqrPol1 = SQR(Binary[Primary].Radius);  /* square of polar radius */
  SqrPol2 = SQR(Binary[Secondary].Radius);

  /* ------------------     eclipse test         ------------------ */

  /* to allow for generalization, we dont insist on                 */
  /*  same number of surface elements on both components            */


  NElem  = IMAX(Binary[Primary].NumPlus,Binary[Secondary].NumPlus);
  NElemP = IMAX(Binary[Primary].NumElem, Binary[Primary].NumPlus);
  NElemS = IMAX(Binary[Secondary].NumElem, Binary[Secondary].NumPlus);

  /*
  NElem  = IMAX(Binary[Primary].NumElem,Binary[Secondary].NumElem);
  NElemP = Binary[Primary].NumElem;
  NElemS = Binary[Secondary].NumElem;
  */

  /* We start at 90 deg, Phase = -0.25, both stars fully visible    */

  if (Flags.elliptic == OFF) {

    Phase = (PI/2 + (2*PI/PhaseSteps) * j ); 
    Orbit.Phase = Phase;
    Orbit.Nu    = Phase;
    M_Phase     = Phase;

  } else {
    
     Phase = Orbit.Phase;
     /* Orbit.OmegaZero is mean anomaly at secondary eclipse        */
     M_Phase = Orbit.M + PI + Orbit.OmegaZero;
  }

  FluxOut[j].Phase = (float) M_Phase;

  SinPhase = sin(Phase); CosPhase = cos(Phase);

  /*
  fprintf(stderr, _("#FIXME   %8.3f   %8.3f   %8.3f   %8.3f   %8.3f   %8.3f   %8.3f   %8.3f  (solar radius)\n"),
	  Orbit.Dist,
	  Orbit.Dist*Orbit.MinDist,
	  Orbit.TrueDistance / SOL_RADIUS,
	  Orbit.Dist * Binary[Primary].Radius * PERIDIST / SOL_RADIUS,
	  Binary[Primary].Radius * Orbit.TrueDistance / SOL_RADIUS, 
	  Orbit.Dist * Binary[Primary].Xp * PERIDIST / SOL_RADIUS,
	  Binary[Primary].Xp * Orbit.TrueDistance / SOL_RADIUS, 
	  Binary[Primary].RadMean * PERIDIST / SOL_RADIUS);
  */

  /* vector towards observer (LOS = Line Of Sight)                  */
  lzero   =  Sini * CosPhase;
  mzero   =  Sini * SinPhase;
  nzero   =  Cosi;

  /* Secondary turn by PI and mirror y->(-y)                        */
  /* this is equivalent to mirror by x->(-x)                        */

  Phase2 = (PI + Phase);
  SinPhase2 = sin(Phase2); CosPhase2 = cos(Phase2);

  /* vector towards observer (LOS = Line Of Sight)                  */
  lzero2   =  Sini * CosPhase2;
  mzero2   =  Sini * SinPhase2;
  nzero2   =  Cosi;

  /* test whether eclipse of Primary definitely impossible          */
  
  /* this flag is (Flags.InEclipse1) required for fractional visibility
   */
  if ( CosPhase <= (CosPhiLagrange-DBL_EPSILON) ) {
    Flags.InEclipse1 = OFF;
  }  else { 
    Flags.InEclipse1 = ON;
  }
  

  /* test whether eclipse of Secondary definitely impossible        */
  
  /* this flag (Flags.InEclipse2) is required for fractional visibility
   */
  if ( CosPhase2 <= (CosPhiLagrange-DBL_EPSILON) ) {
    Flags.InEclipse2 = OFF;
  }  else { 
    Flags.InEclipse2 = ON;
  }
  

  SurfPtrP = Surface[Primary];
  SurfPtrS = Surface[Secondary];

  for (i = 0; i < NElem; ++i) {      /* loop over surface elements */

    /* START WITH PRIMARY                                          */ 

    if (i < NElemP) {

      SurfPtrP->Velocity = 
	- OmegaDotPrimary * Orbit.TrueDistance * Orbit.Dist /* FIXME */
	* (-SurfPtrP->mu     * lzero
	   + SurfPtrP->lambda * mzero);


      SurfPtrP->LOS_Pot = 0.0;
      /* angle surface normal - LOS                                */
      Cosgamma =   lzero * SurfPtrP->l
	+          mzero * SurfPtrP->m
	+          nzero * SurfPtrP->n;

      /* IF (Cosgamma > 0) THEN Visible Side, Test for Eclipse     */
      if (Cosgamma <= 0) 
	eclipsed = Invisible;  /* invisible                        */
      if (Cosgamma >= DBL_EPSILON){   /* Visible Side              */

	eclipsed = Yes;
	++np_visible;

	/* test whether eclipse definitely impossible              */
	if (CosPhase <= (CosPhiLagrange-DBL_EPSILON)) eclipsed = No;

	if (Flags.fill == ON) {
	  if (i > Binary[Primary].NumElem) eclipsed = Yes;
	}

	/* IF (eclipsed == Yes) MORE TESTING                       */
	if (eclipsed == Yes) { 

	  /* intersect with circle of rad Xp2 centered on second   */
	  /* -- minimum sphere containing the star                 */
	  x_z = SurfPtrP->lambda;
	  y_z = SurfPtrP->mu;
	  z_z = SurfPtrP->nu;
	  Qi  = 2.0*( x_z*lzero + y_z*mzero + z_z*nzero - lzero);
	  Li  = 1.0 + x_z*x_z + y_z*y_z + z_z*z_z - 2*x_z - SqrXp2;
	  QLi = Qi*Qi - 4.0*Li;
	  if (QLi <= 0.0) {    /* No intersection                  */ 
	    eclipsed = No; 
	  } else {             /* Intersection, next test          */

            /* check whether distance is greater than polar radius */
            /* -- maximum inscribed sphere                         */
	    t_l = sqrt(QLi);    /* just to hold temporary result   */
	    t_1 = (-Qi + t_l)/2.0;
	    t_2 = (-Qi - t_l)/2.0;
	    t_l = (t_1 + t_2)/2.0; 
	    /* t_l is Midpoint of ray intersecting Xp2-Sphere      */
	    x_tl = x_z + t_l * lzero;
	    y_tl = y_z + t_l * mzero;
	    z_tl = z_z + t_l * nzero;
	    t_dist = SQR(x_tl-1.0) + y_tl * y_tl + z_tl * z_tl;

	    if (Flags.fill == ON) {
	      if (i > Binary[Primary].NumElem) { 
		/* rw 06.02.2007 */
		/* eclipsed = Yes; */ /* superfluous */
		/* incorrect, will test against wrong star
		 * t_1 = MAX(t_1, t_2);
		 * t_2 = t_1 + 1.0;
		 */

		/* next test would yield incorrect result, thus jump
		 * to the MinFinder() routine
		 */
		t_dist =  SqrPol2+1.; 
	      }
	    }

	    if (t_dist <= (SqrPol2-DBL_EPSILON)) { 

	      eclipsed = Yes;       /* definitely eclipsed         */

	     } else {


	       /* Find Potential Minimum between t_1 and t_2       */
               /* final resort                                     */
               /* changed Binary[Pri].Mq to Binary[Sec].Mq Jul 6   */
               MinErr = MinFinder(&Binary[Secondary].Mq, 
				  &fratio2, 
				  &t_1, &t_2, 
				  &lzero, &mzero, &nzero,
				  &x_z, &y_z, &z_z,
				  &tmin, &PotMin);
               if (MinErr == 1 && Flags.fill == OFF) 
		 WARNING(_("LightCurve: No Minimum Found"));
               PotMin = -PotMin; 
               SurfPtrP->LOS_Pot = (float) PotMin;

	       /* added condition tmin >= 0                        */
               if (PotMin >= Pot_Two && tmin >= 0) { 
                 eclipsed = Yes; 
	       } else {
                 eclipsed = No;
	       }

	     }
	  }
	}

	if (eclipsed == Yes) ++np_eclipsed;
	SurfPtrP->EclipseFlag = eclipsed;
	SurfPtrP->CosGamma    = (float) Cosgamma;
	if (eclipsed > 0) SurfPtrP->visibility = 0;
	else SurfPtrP->visibility = 1;

      } else {
	/* invisible side                                          */
	SurfPtrP->EclipseFlag = eclipsed;
	SurfPtrP->CosGamma    = -1;
	if (eclipsed > 0) SurfPtrP->visibility = 0;
	else SurfPtrP->visibility = 1;
      }
      ++SurfPtrP;
    }
    /* END PRIMARY                                                 */

    /* HERE SECONDARY                                              */
    if (i < NElemS) {

      SurfPtrS->Velocity = 
	- OmegaDotSecondary * Orbit.TrueDistance * Orbit.Dist /* FIXME */
	* (SurfPtrS->mu     * lzero2
	   + SurfPtrS->lambda * mzero2);

      SurfPtrS->LOS_Pot = 0.0;
      /* angle surface normal - LOS                                */
      /*  dont forget to mirror in y->(-y)                         */
      Cosgamma =   lzero2 * SurfPtrS->l
	+          mzero2 * (-SurfPtrS->m)
	+          nzero2 * SurfPtrS->n;

      /* IF (Cosgamma > 0) THEN Visible Side, Test for Eclipse     */
      if (Cosgamma <= 0) 
	eclipsed = Invisible;  /* invisible                        */
      if (Cosgamma >= DBL_EPSILON){   /* Visible Side              */

	eclipsed = Yes;
	++ns_visible;

	/* test whether eclipse definitely impossible              */
	if (CosPhase2 <= (CosPhiLagrange-DBL_EPSILON)) eclipsed = No; 

	if (Flags.fill == ON) {
	  if (i > Binary[Secondary].NumElem) eclipsed = Yes;
	}

	/* IF (eclipsed == Yes) MORE TESTING                       */ 
	if (eclipsed == Yes) { 

	  /* intersect with circle of rad Xp1 centered on prim     */
	  /* -- minimum sphere containing the star                 */
	  x_z = SurfPtrS->lambda;
	  y_z = (-SurfPtrS->mu);
	  z_z = SurfPtrS->nu;
	  Qi  = 2.0*( x_z*lzero2 + y_z*mzero2 + z_z*nzero2 - lzero2);
	  Li  = 1.0 + x_z*x_z + y_z * y_z + z_z*z_z - 2*x_z - SqrXp1;
	  QLi = Qi*Qi - 4.0*Li;
	  if (QLi <= 0.0) {    /* No intersection                  */ 
	    eclipsed = No; 
	  } else {             /* Intersection, next test          */

            /* check whether distance is greater than polar radius */
            /* -- maximum inscribed sphere                         */
	    t_l = sqrt(QLi);    /* just to hold temporary result   */
	    t_1 = (-Qi + t_l)/2.0;
	    t_2 = (-Qi - t_l)/2.0;
	    t_l = (t_1 + t_2)/2.0; 
	    /* t_l is Midpoint of ray intersecting Xp2-Sphere      */
	    x_tl = x_z + t_l * lzero2;
	    y_tl = y_z + t_l * mzero2;
	    z_tl = z_z + t_l * nzero2;
	    t_dist = SQR(x_tl-1.0) + y_tl*y_tl + z_tl*z_tl;

	    if (Flags.fill == ON) {
	      if (i > Binary[Secondary].NumElem) {
		/* rw 06.02.2007 */
		/* eclipsed = Yes; */ /* superfluous */
		/* incorrect, will test against wrong star
		 * t_1 = MAX(t_1, t_2);
		 * t_2 = t_1 + 1.0;
		 */

		/* next test would yield incorrect result, thus jump
		 * to the MinFinder() routine
		 */
		t_dist =  SqrPol1+1.;
	      }
	    }

	    /* we do not need the sqrt, compare the squares        */
	    if (t_dist <= (SqrPol1-DBL_EPSILON)) { 

	      eclipsed = Yes;       /* definitely eclipsed         */

	    } else {

	      /* Find Potential Minimum between t_1 and t_2        */
	      /* final resort                                      */
	      /* changed Binary[Sec].Mq to Binary[Pri].Mq Jul 6    */
	      MinErr = MinFinder(&Binary[Primary].Mq, 
				 &fratio1, 
				 &t_1, &t_2, 
				 &lzero2, &mzero2, &nzero2,
				 &x_z, &y_z, &z_z,
				 &tmin, &PotMin);
	      if (MinErr == 1) WARNING(_("LightCurve: No Minimum Found"));
	      PotMin = -PotMin; 
	      SurfPtrS->LOS_Pot = (float) PotMin;

	       /* added condition tmin >= 0                        */
	      if (PotMin >= (Pot_One-DBL_EPSILON) && tmin >= 0) { 
		eclipsed = Yes; 
	      } else {
		eclipsed = No;
	      }


	    }
	  }
	}

	if (eclipsed == Yes) ++ns_eclipsed;
	SurfPtrS->EclipseFlag = eclipsed;
	SurfPtrS->CosGamma    = (float) Cosgamma;
	if (eclipsed > 0) SurfPtrS->visibility = 0;
	else SurfPtrS->visibility = 1;

      } else {
	SurfPtrS->EclipseFlag = eclipsed;
	SurfPtrS->CosGamma    = -1;
	if (eclipsed > 0) SurfPtrS->visibility = 0;
	else SurfPtrS->visibility = 1;
      }
      ++SurfPtrS;
    }
    /* END  SECONDARY                                              */


  } /* end loop over surface elements                              */

  M_Phase = Orbit.M + PI + Orbit.OmegaZero;

  contactflag_old = contactflag;

  if (ns_eclipsed > 0) 
    contactflag |= secondary_eclipsed;
  else
    contactflag &= ~secondary_eclipsed;

  if (ns_eclipsed == ns_visible) 
    contactflag |= secondary_total;
  else
    contactflag &= ~secondary_total;

  if (np_eclipsed > 0) 
    contactflag |= primary_eclipsed;
  else
    contactflag &= ~primary_eclipsed;

  if (np_eclipsed == np_visible) 
    contactflag |= primary_total;
  else
    contactflag &= ~primary_total;

  if ((contactflag & secondary_eclipsed) != (contactflag_old & secondary_eclipsed)) {
    if (contactflag & secondary_eclipsed) {
      if (Flags.debug[VERBOSE] == ON) 
	printf(_(" Secondary  eclipse first contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[1] = M_Phase;
    }
    else {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Secondary eclipse fourth contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[2] = M_Phase;
    }
  }

  if ((contactflag & secondary_total) != (contactflag_old & secondary_total)) {
    if (contactflag & secondary_total) {
      if (Flags.debug[VERBOSE] == ON) 
	printf(_(" Secondary  eclipse second contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[5] = M_Phase;
    }
    else {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Secondary eclipse third contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[6] = M_Phase;
    }
  }

  if ((contactflag & primary_eclipsed) != (contactflag_old & primary_eclipsed)) {
    if (contactflag & primary_eclipsed) {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Primary  eclipse first contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[3] = M_Phase;
    }
    else {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Primary eclipse fourth contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[4] = M_Phase;
    }
  }

  if ((contactflag & primary_total) != (contactflag_old & primary_total)) {
    if (contactflag & primary_total) {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Primary  eclipse second contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[7] = M_Phase;
    }
    else {
      if (Flags.debug[VERBOSE] == ON)
	printf(_(" Primary eclipse third contact at phase: %8.6f\n"), Phase);
      Orbit.Contact[8] = M_Phase;
    }
  }

  return;
} /* end LightCurve                                                */

/* From an article by Mark Lowne / mark marklowne com
 * This allows a branch-free "if (x >= 0) {...}".
 * No speedup though on the "if (CosGamma >= 0) ..."
 */
inline float ispositive(float x) 
{
  union {
    float f;
    int i;
  } v;
  v.f = x;
  v.i = ((~v.i) >> 8) & 0x3f800000;
  
  return v.f;
}


/* #define DEBUG_FLUX 1 */

#ifdef DEBUG_FLUX
double * flux_comp[NUM_COMP] = { NULL, NULL, NULL };

inline void init_dbg_flux(int Comp)
{
  flux_comp[Comp]    = calloc(MAXELEMENTS, sizeof(double));
  return;
}
inline void destroy_dbg_flux(int Comp)
{
  if (flux_comp[Comp] != NULL)
    free (flux_comp[Comp]);
  flux_comp[Comp]    = NULL;
  return;
}
inline void print_dbg_flux(int Comp)
{
  int nelem         = Binary[Comp].NumElem;
  int i;
  char name[64];
  FILE * f;

  snprintf(name, 64, "Nightfall_dbg_flux_%d", Comp);
  name[63] = '\0';
  
  f = fopen(name, "w");
  if (f) {
    if ((flux_comp[Comp] != NULL) && nelem > 0) {
      for (i = 0; i < PhaseSteps; ++i) {
	fprintf(f, "%d, %d, %12.6e\n", Comp, i, flux_comp[Comp][i]);
      }
    }
    fclose(f);
  }
  destroy_dbg_flux(Comp);
  return;
}
#else
static double * flux_comp[NUM_COMP] = { NULL, NULL, NULL };
inline void init_dbg_flux(int Comp) { (void) Comp; }
inline void destroy_dbg_flux(int Comp){ (void) Comp; }
inline void print_dbg_flux(int Comp) { (void) Comp; }

#endif

/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Flux computation
 @param     (int)  Comp  The stellar component 
 @param     (int)  Phase The phase index 
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void LightFlux(int Comp, int Phase)
{
  register long           i;                 /* loop counter        */
  double                  Cosgamma;          /* LOS angle           */
  double                  Common;            /* common part         */
  double                  Weight;            /* weight factor       */
  double                  SumOfVelocity;     /* normalization       */
  double                  SumOfWeight;       /* normalization       */
  double                  OneMinCosgamma;    /* 1.0 - Cosgamma      */
  SurfaceElement          *SurfPtr;          /* pointer to surface  */
  long                    nelem;             /* loop limit          */
  PhotoOut                *OFlux;            /* pointer to outflux  */

  /* ------------------ flux computing              --------------- */

  /* We start at 90 deg, Phase = 0.25, both stars fully visible     */

  /* loops over passbands are unrolled, all if's in loop eliminated */

  SumOfVelocity = 0.0;
  SumOfWeight   = 0.0;
  nelem         = Binary[Comp].NumElem;
  OFlux         = &FluxOut[Phase];
  SurfPtr       = Surface[Comp];

  if (flux_comp[Comp] == NULL)
    init_dbg_flux(Comp);
  
  if (Flags.limb == 0) {

    for (i = 0; i < nelem; ++i) {      

      /* loop over surface elements                                   */
      /* add up Flux of visible elements                              */
      /* IF NOT eclipsed THEN                                         */

      if (SurfPtr->CosGamma >= 0.0) {
	Cosgamma = SurfPtr->CosGamma;
	Common   = SurfPtr->visibility 
	  * Cosgamma;

	OneMinCosgamma = (1.0 - Cosgamma);
	
	Weight = SurfPtr->f_[0] * Common
	  * (1.0 - Limb[Comp][0][0]*OneMinCosgamma);
	OFlux->Mag[0] = OFlux->Mag[0] + Weight;

	Weight = SurfPtr->f_[1] * Common
	  * (1.0 - Limb[Comp][1][0]*OneMinCosgamma);
	OFlux->Mag[1] = OFlux->Mag[1] + Weight;

	Weight = SurfPtr->f_[2] * Common
	  * (1.0 - Limb[Comp][2][0]*OneMinCosgamma);
	OFlux->Mag[2] = OFlux->Mag[2] + Weight;

#ifdef DEBUG_FLUX
	flux_comp[Comp][Phase] += Weight;
#endif

	SumOfVelocity        = SumOfVelocity
	  + Weight * SurfPtr->Velocity;
	SumOfWeight          = SumOfWeight + Weight;
	
	Weight = SurfPtr->f_[3] * Common
	  * (1.0 - Limb[Comp][3][0]*OneMinCosgamma);
	OFlux->Mag[3] = OFlux->Mag[3] + Weight;

	Weight = SurfPtr->f_[4] * Common
	  * (1.0 - Limb[Comp][4][0]*OneMinCosgamma);
	OFlux->Mag[4] = OFlux->Mag[4] + Weight;

	Weight = SurfPtr->f_[5] * Common
	  * (1.0 - Limb[Comp][5][0]*OneMinCosgamma);
	OFlux->Mag[5] = OFlux->Mag[5] + Weight;

	Weight = SurfPtr->f_[6] * Common
	  * (1.0 - Limb[Comp][6][0]*OneMinCosgamma);
	OFlux->Mag[6] = OFlux->Mag[6] + Weight;

	Weight = SurfPtr->f_[7] * Common
	  * (1.0 - Limb[Comp][7][0]*OneMinCosgamma);
	OFlux->Mag[7] = OFlux->Mag[7] + Weight;

	Weight = SurfPtr->f_[8] * Common
	  * (1.0 - Limb[Comp][8][0]*OneMinCosgamma);
	OFlux->Mag[8] = OFlux->Mag[8] + Weight;

	Weight = SurfPtr->f_[9] * Common
	  * (1.0 - Limb[Comp][9][0]*OneMinCosgamma);
	OFlux->Mag[9] = OFlux->Mag[9] + Weight;

	Weight = SurfPtr->f_[10] * Common
	  * (1.0 - Limb[Comp][10][0]*OneMinCosgamma);
	OFlux->Mag[10] = OFlux->Mag[10] + Weight;

	Weight = SurfPtr->f_[11] * Common
	  * (1.0 - Limb[Comp][10][0]*OneMinCosgamma);
	OFlux->Mag[11] = OFlux->Mag[11] + Weight;
	
      }
      ++SurfPtr;
    }
  }

  else if ( Flags.limb == 1) {

    double OneMinCosgamma2;

    for (i = 0; i < nelem; ++i) {      
      
      /* loop over surface elements                                   */
      /* add up Flux of visible elements                              */
      /* IF NOT eclipsed THEN                                         */
      
      if (SurfPtr->CosGamma >= 0.0) {

	Cosgamma = SurfPtr->CosGamma;
	Common   = SurfPtr->visibility 
	  * Cosgamma;
	OneMinCosgamma = (1.0 - Cosgamma);
	OneMinCosgamma2 = OneMinCosgamma*OneMinCosgamma;

	Weight = SurfPtr->f_[0] * Common
	    *(1.0 - Limb[Comp][0][0]*OneMinCosgamma
	      - Limb[Comp][0][1]*OneMinCosgamma2);
	OFlux->Mag[0] = OFlux->Mag[0] + Weight;
	Weight = SurfPtr->f_[1] * Common
	    *(1.0 - Limb[Comp][1][0]*OneMinCosgamma
	      - Limb[Comp][1][1]*OneMinCosgamma2);
	OFlux->Mag[1] = OFlux->Mag[1] + Weight;
	Weight = SurfPtr->f_[2] * Common
	    *(1.0 - Limb[Comp][2][0]*OneMinCosgamma
	      - Limb[Comp][2][1]*OneMinCosgamma2);
	OFlux->Mag[2] = OFlux->Mag[2] + Weight;

#ifdef DEBUG_FLUX
	flux_comp[Comp][Phase] += Weight;
#endif

	SumOfVelocity        = SumOfVelocity
	  + Weight * SurfPtr->Velocity;
	SumOfWeight          = SumOfWeight + Weight;

	Weight = SurfPtr->f_[3] * Common
	    *(1.0 - Limb[Comp][3][0]*OneMinCosgamma
	      - Limb[Comp][3][1]*OneMinCosgamma2);
	OFlux->Mag[3] = OFlux->Mag[3] + Weight;
	Weight = SurfPtr->f_[4] * Common
	    *(1.0 - Limb[Comp][4][0]*OneMinCosgamma
	      - Limb[Comp][4][1]*OneMinCosgamma2);
	OFlux->Mag[4] = OFlux->Mag[4] + Weight;
	Weight = SurfPtr->f_[5] * Common
	    *(1.0 - Limb[Comp][5][0]*OneMinCosgamma
	      - Limb[Comp][5][1]*OneMinCosgamma2);
	OFlux->Mag[5] = OFlux->Mag[5] + Weight;
	Weight = SurfPtr->f_[6] * Common
	    *(1.0 - Limb[Comp][6][0]*OneMinCosgamma
	      - Limb[Comp][6][1]*OneMinCosgamma2);
	OFlux->Mag[6] = OFlux->Mag[6] + Weight;
	Weight = SurfPtr->f_[7] * Common
	    *(1.0 - Limb[Comp][7][0]*OneMinCosgamma
	      - Limb[Comp][7][1]*OneMinCosgamma2);
	OFlux->Mag[7] = OFlux->Mag[7] + Weight;
	Weight = SurfPtr->f_[8] * Common
	    *(1.0 - Limb[Comp][8][0]*OneMinCosgamma
	      - Limb[Comp][8][1]*OneMinCosgamma2);
	OFlux->Mag[8] = OFlux->Mag[8] + Weight;
	Weight = SurfPtr->f_[9] * Common
	    *(1.0 - Limb[Comp][9][0]*OneMinCosgamma
	      - Limb[Comp][9][1]*OneMinCosgamma2);
	OFlux->Mag[9] = OFlux->Mag[9] + Weight;
	Weight = SurfPtr->f_[10] * Common
	    *(1.0 - Limb[Comp][10][0]*OneMinCosgamma
	      - Limb[Comp][10][1]*OneMinCosgamma2);
	OFlux->Mag[10] = OFlux->Mag[10] + Weight;
	Weight = SurfPtr->f_[11] * Common
	    *(1.0 - Limb[Comp][11][0]*OneMinCosgamma
	      - Limb[Comp][11][1]*OneMinCosgamma2);
	OFlux->Mag[11] = OFlux->Mag[11] + Weight;
	
      }
      ++SurfPtr;
    }
  }
  else if ( Flags.limb == 2) {

    double sqrtCosgamma;

    for (i = 0; i < nelem; ++i) {      

      /* loop over surface elements                                   */
      /* add up Flux of visible elements                              */
      /* IF NOT eclipsed THEN                                         */

      if (SurfPtr->CosGamma >= 0.0) {

	Cosgamma = SurfPtr->CosGamma;
	Common   = SurfPtr->visibility 
	  * Cosgamma;
	OneMinCosgamma = (1.0 - Cosgamma);
	sqrtCosgamma   = 1.0 - sqrt(Cosgamma);

	Weight = SurfPtr->f_[0] * Common
	    *(1.0 - Limb[Comp][0][0]*OneMinCosgamma
	      - Limb[Comp][0][1]*sqrtCosgamma);
	OFlux->Mag[0] = OFlux->Mag[0] + Weight;
	Weight = SurfPtr->f_[1] * Common
	    *(1.0 - Limb[Comp][1][0]*OneMinCosgamma
	      - Limb[Comp][1][1]*sqrtCosgamma);
	OFlux->Mag[1] = OFlux->Mag[1] + Weight;
	Weight = SurfPtr->f_[2] * Common
	    *(1.0 - Limb[Comp][2][0]*OneMinCosgamma
	      - Limb[Comp][2][1]*sqrtCosgamma);
	OFlux->Mag[2] = OFlux->Mag[2] + Weight;

#ifdef DEBUG_FLUX
	flux_comp[Comp][Phase] += Weight;
#endif

	SumOfVelocity        = SumOfVelocity
	  + Weight * SurfPtr->Velocity;
	SumOfWeight          = SumOfWeight + Weight;
	
	Weight = SurfPtr->f_[3] * Common
	    *(1.0 - Limb[Comp][3][0]*OneMinCosgamma
	      - Limb[Comp][3][1]*sqrtCosgamma);
	OFlux->Mag[3] = OFlux->Mag[3] + Weight;
	Weight = SurfPtr->f_[4] * Common
	    *(1.0 - Limb[Comp][4][0]*OneMinCosgamma
	      - Limb[Comp][4][1]*sqrtCosgamma);
	OFlux->Mag[4] = OFlux->Mag[4] + Weight;
	Weight = SurfPtr->f_[5] * Common
	    *(1.0 - Limb[Comp][5][0]*OneMinCosgamma
	      - Limb[Comp][5][1]*sqrtCosgamma);
	OFlux->Mag[5] = OFlux->Mag[5] + Weight;
	Weight = SurfPtr->f_[6] * Common
	    *(1.0 - Limb[Comp][6][0]*OneMinCosgamma
	      - Limb[Comp][6][1]*sqrtCosgamma);
	OFlux->Mag[6] = OFlux->Mag[6] + Weight;
	Weight = SurfPtr->f_[7] * Common
	    *(1.0 - Limb[Comp][7][0]*OneMinCosgamma
	      - Limb[Comp][7][1]*sqrtCosgamma);
	OFlux->Mag[7] = OFlux->Mag[7] + Weight;
	Weight = SurfPtr->f_[8] * Common
	    *(1.0 - Limb[Comp][8][0]*OneMinCosgamma
	      - Limb[Comp][8][1]*sqrtCosgamma);
	OFlux->Mag[8] = OFlux->Mag[8] + Weight;
	Weight = SurfPtr->f_[9] * Common
	    *(1.0 - Limb[Comp][9][0]*OneMinCosgamma
	      - Limb[Comp][9][1]*sqrtCosgamma);
	OFlux->Mag[9] = OFlux->Mag[9] + Weight;
	Weight = SurfPtr->f_[10] * Common
	    *(1.0 - Limb[Comp][10][0]*OneMinCosgamma
	      - Limb[Comp][10][1]*sqrtCosgamma);
	OFlux->Mag[10] = OFlux->Mag[10] + Weight;
	Weight = SurfPtr->f_[11] * Common
	    *(1.0 - Limb[Comp][11][0]*OneMinCosgamma
	      - Limb[Comp][11][1]*sqrtCosgamma);
	OFlux->Mag[11] = OFlux->Mag[11] + Weight;
	
      }
      ++SurfPtr;
    }
  }
  else if ( Flags.limb == 3) {

    double sqrtCosgamma;
    double Cosgamma2;
    double Cosgamma32;

    for (i = 0; i < nelem; ++i) {      

      /* loop over surface elements                                   */
      /* add up Flux of visible elements                              */
      /* IF NOT eclipsed THEN                                         */

      if (SurfPtr->CosGamma >= 0.0) {

	Cosgamma = SurfPtr->CosGamma;
	Common   = SurfPtr->visibility 
	  * Cosgamma;

	sqrtCosgamma   = 1.0 - sqrt(Cosgamma);
	OneMinCosgamma = 1.0 - Cosgamma;
	Cosgamma32     = 1.0 - Cosgamma*sqrt(Cosgamma);
	Cosgamma2      = 1.0 - Cosgamma*Cosgamma;
	
	Weight = SurfPtr->f_[0] * Common
	    *(1.0 - Limb[Comp][0][0]*sqrtCosgamma - Limb[Comp][0][1]*OneMinCosgamma
	      - Limb[Comp][0][2]*Cosgamma32 - Limb[Comp][0][3]*Cosgamma2);
	OFlux->Mag[0] = OFlux->Mag[0] + Weight;
	Weight = SurfPtr->f_[1] * Common
	    *(1.0 - Limb[Comp][1][0]*sqrtCosgamma - Limb[Comp][1][1]*OneMinCosgamma
	      - Limb[Comp][1][2]*Cosgamma32 - Limb[Comp][1][3]*Cosgamma2);
	OFlux->Mag[1] = OFlux->Mag[1] + Weight;
	Weight = SurfPtr->f_[2] * Common
	    *(1.0 - Limb[Comp][2][0]*sqrtCosgamma - Limb[Comp][2][1]*OneMinCosgamma
	      - Limb[Comp][2][2]*Cosgamma32 - Limb[Comp][2][3]*Cosgamma2);
	OFlux->Mag[2] = OFlux->Mag[2] + Weight;

#ifdef DEBUG_FLUX
	flux_comp[Comp][Phase] += Weight;
#endif

	SumOfVelocity        = SumOfVelocity
	  + Weight * SurfPtr->Velocity;
	SumOfWeight          = SumOfWeight + Weight;
	
	Weight = SurfPtr->f_[3] * Common
	    *(1.0 - Limb[Comp][3][0]*sqrtCosgamma - Limb[Comp][3][1]*OneMinCosgamma
	      - Limb[Comp][3][2]*Cosgamma32 - Limb[Comp][3][3]*Cosgamma2);
	OFlux->Mag[3] = OFlux->Mag[3] + Weight;
	Weight = SurfPtr->f_[4] * Common
	    *(1.0 - Limb[Comp][4][0]*sqrtCosgamma - Limb[Comp][4][1]*OneMinCosgamma
	      - Limb[Comp][4][2]*Cosgamma32 - Limb[Comp][4][3]*Cosgamma2);
	OFlux->Mag[4] = OFlux->Mag[4] + Weight;
	Weight = SurfPtr->f_[5] * Common
	    *(1.0 - Limb[Comp][5][0]*sqrtCosgamma - Limb[Comp][5][1]*OneMinCosgamma
	      - Limb[Comp][5][2]*Cosgamma32 - Limb[Comp][5][3]*Cosgamma2);
	OFlux->Mag[5] = OFlux->Mag[5] + Weight;
	Weight = SurfPtr->f_[6] * Common
	    *(1.0 - Limb[Comp][6][0]*sqrtCosgamma - Limb[Comp][6][1]*OneMinCosgamma
	      - Limb[Comp][6][2]*Cosgamma32 - Limb[Comp][6][3]*Cosgamma2);
	OFlux->Mag[6] = OFlux->Mag[6] + Weight;
	Weight = SurfPtr->f_[7] * Common
	    *(1.0 - Limb[Comp][7][0]*sqrtCosgamma - Limb[Comp][7][1]*OneMinCosgamma
	      - Limb[Comp][7][2]*Cosgamma32 - Limb[Comp][7][3]*Cosgamma2);
	OFlux->Mag[7] = OFlux->Mag[7] + Weight;
	Weight = SurfPtr->f_[8] * Common
	    *(1.0 - Limb[Comp][8][0]*sqrtCosgamma - Limb[Comp][8][1]*OneMinCosgamma
	      - Limb[Comp][8][2]*Cosgamma32 - Limb[Comp][8][3]*Cosgamma2);
	OFlux->Mag[8] = OFlux->Mag[8] + Weight;
	Weight = SurfPtr->f_[9] * Common
	    *(1.0 - Limb[Comp][9][0]*sqrtCosgamma - Limb[Comp][9][1]*OneMinCosgamma
	      - Limb[Comp][9][2]*Cosgamma32 - Limb[Comp][9][3]*Cosgamma2);
	OFlux->Mag[9] = OFlux->Mag[9] + Weight;
	Weight = SurfPtr->f_[10] * Common
	    *(1.0 - Limb[Comp][10][0]*sqrtCosgamma - Limb[Comp][10][1]*OneMinCosgamma
	      - Limb[Comp][10][2]*Cosgamma32 - Limb[Comp][10][3]*Cosgamma2);
	OFlux->Mag[10] = OFlux->Mag[10] + Weight;
	Weight = SurfPtr->f_[11] * Common
	    *(1.0 - Limb[Comp][11][0]*sqrtCosgamma - Limb[Comp][11][1]*OneMinCosgamma
	      - Limb[Comp][11][2]*Cosgamma32 - Limb[Comp][11][3]*Cosgamma2);
	OFlux->Mag[11] = OFlux->Mag[11] + Weight;
	
      }
      ++SurfPtr;
    }
  }

  if (Phase == (PhaseSteps - 1))
    print_dbg_flux(Comp);

  /* ------------  normalize velocity ----------------------        */

  if (SumOfWeight >= DBL_EPSILON) {
    FluxOut[Phase].D_RV[Comp] = (float) (SumOfVelocity / SumOfWeight);
  } else {
    FluxOut[Phase].D_RV[Comp] = 0.0;
  }

  if (Flags.elliptic == ON)
    FluxOut[Phase].D_RV[Comp] = (-1.0) * FluxOut[Phase].D_RV[Comp];

  FluxOut[Phase].RV[Comp] = 
    FluxOut[Phase].D_RV[Comp] + FluxOut[Phase].RV[Comp];

  return;
}

/****************************************************************************
 @package   nightfall
 @author    Rainer Wichmann (rwichman@lsw.uni-heidelberg.de)
 @version   1.0
 @short     Find minimum of the potential along some line-of-sight vector.
 @long      Uses Brents algorithm, which is a combination of golden section 
            search and parabolic interpolation.
            Adopted from procedure 'localmin' in: Richard Brent, Algorithms for
            minimization without derivatives, Prentice-Hall, Inc. (1973)
 @tip       Input is a parametric form of LOS vector + brackets.
 @param     (double*) q  Mass ratio
 @param     (double*) ff Async rotation
 @param     (double*) t1 Minimum bracket
 @param     (double*) t3 Minimum bracket
 @param     (double*) l0 x-component LOS vector
 @param     (double*) m0 y-component LOS vector
 @param     (double*) n0 z-component LOS vector
 @param     (double*) x0 x-component of point on LOS vector
 @param     (double*) y0 y-component of point on LOS vector
 @param     (double*) z0 z-component of point on LOS vector
 @param     (double*)  tmin The position of the minimum
 @param     (double*)  Cmin The LOS potential minimum
 @return    (int)    The error status
 @heading   Light Curve
 ****************************************************************************/
int MinFinder(double *q, double *ff, double *t1, double *t3, 
	      double *l0, double *m0, double *n0,
	      double *x0, double *y0, double *z0,
	      double *tmin, /*@out@*/ double *Cmin)
{
  register      int i;                 /* loop variable                     */
  double        t2;                    /* middle value                      */
  double        tmp;                   /* temp var                          */
  double        step, stepstore;       /* stepsizes                         */
  double        Fe = 0.0, Ff = 0.0;    /* function values                   */
  double        Fg = 0.0, Fh = 0.0;    /* function values                   */
  double        e = 0.0, f, g, h;      /* positions (parametric)            */
  double        delta1, delta2;        /* test convergence                  */
  double        Start, End, Middle;    /* brackets                          */
  double        nextstep = 0.0;        /* stepsize                          */
  double        x, y, z;               /* position in cartesian coordinates */
  double        P1 = 0.0, P2, P3;      /* fit parabola                      */ 
  double        precise = 0.01;        /* convergence criterion             */
  double        x2, y2, z2;            /* squared values                    */

  if ( (*t1) <= ((*t3)-DBL_EPSILON) ) { 
    Start = (*t1); 
    End = (*t3); 
  } else { 
    Start = (*t3); 
    End = (*t1); 
  }

  step = 0.0;

  /* we have a parametric description of a vector X(h)                      */
  /* we want to find min(func) = min(X(h)) as function of h                 */

  t2 = 0.62*(*t1) + 0.38*(*t3);

  /* initial value of X(h)                                                  */
  x = 1.0 - ((*x0) + (*l0)*(t2));  
  y =       -(*y0) - (*m0)*(t2);
  z =        (*z0) + (*n0)*(t2);

  x2 = x*x;
  y2 = y*y;
  z2 = z*z;

  /* NEW jul 6,98 -- change of coordinate system, calculate                 */
  /* in system of eclipsing star                                            */
  /* (i.e. x = 1.0 - x; y = -y; z = z;)                                     */

  /* t2 is the initial guess for the minimum between t1 and t3              */
  h = g = f = t2;

  /* initial value of F                                                     */
  tmp = 1.0-x; 
  tmp = sqrt(tmp*tmp + y2 + z2 );
  Ff = Fg = Fh = -(  1.0/sqrt(x2 + y2 + z2) 
                   + (*q) * (1.0/tmp - x)
                   + ( x2 + y2 ) * (*ff)*(*ff) * (0.5*( (*q) + 1.0)));

  /* we do a maximum of MAXITER iterations - convergence usually            */
  /*  after less than ten iterations                                        */
  /*@i@*/ for (i = MAXITER; i; --i) {

    Middle = 0.5 * (Start + End);

    delta1 = precise * fabs(h) + DBL_EPSILON;
    delta2 = 2.0 * delta1;

    /* test for convergence                                                 */
    /* RETURN if converged                                                  */

    if (fabs(h-Middle) <= (delta2 - 0.5 * (End-Start) ) ) {
      *tmin = h;
      *Cmin = Fh;
      return(0);
    }

    if (fabs(step) >= delta1) {

      /* fit parabola                                                       */
      P3 = (h-g)*(Fh-Ff);
      P2 = (h-f)*(Fh-Fg);
      P1 = (h-f)*P2 - (h-g)*P3;
      P2 = 2.0 * (P2 - P3);

      if (P2 >= DBL_EPSILON) { 
        P1 = -P1; 
      } else {
        P2 = -P2;
      }

      stepstore = step;
      step = nextstep;

      if ( (fabs(P1) >= fabs(0.5 * P2 * stepstore)) 
           || (P1 <= P2*(Start-h)) 
           || (P1 >= P2*(End-h))) {

        /* a golden-section step                                            */
        if (h >= Middle) { 
          step = (Start-h); 
        } else {  
          step = (End-h);
        }
        nextstep = 0.381966 * step;

      } else {

        /* do a parabolic interpolation step                                */
        nextstep = P1/P2;
        e        = h + nextstep;

        /* f must not be evaluated too close to Start or End                */
        if ((e-Start) <= delta2 || (End-e) <= delta2) {
          if ((Middle-h) <= (-DBL_EPSILON)) { 
            nextstep = nextstep-delta1; 
          } else { 
            nextstep = nextstep+delta1;
          }
        }

      }

    } else {

      /* a golden-section step                                              */
      if (h >= Middle) { 
        step = (Start-h); 
      } else {  
        step = (End-h);
      }
      nextstep =  0.381966 * step;
      
    }

    if (fabs(nextstep) >= delta1) { 
      e = h + nextstep; 
    } else {
      /* f must not be evaluated too close                                  */
      if ( nextstep <= (-DBL_EPSILON) ) { 
        e = h - delta1; 
      } else { 
        e = h + delta1; 
      }
    }

    /* e ist the current best guess for the minimum                         */
    /* calculate X(e)                                                       */
    x = 1.0 - ((*x0) + (*l0) * e);  
    y =     -  (*y0) - (*m0) * e;  
    z =        (*z0) + (*n0) * e;

    x2 = x*x;
    y2 = y*y;
    z2 = z*z;

    /* NEW jul 6 -- change of coordinate system, calculate                  */
    /* in system of eclipsing star                                          */
    /* i.e x = 1.0 - x;  y = -y;  z = z;                                    */

    /* calculate f(X(e))                                                    */
    tmp = 1.0-x;
    tmp = sqrt(tmp*tmp + y2 + z2 );
    Fe         = -(  1.0/sqrt(x2 + y2 + z2) 
                   + (*q) * (1.0/tmp - x)
                   + ( x2 + y2 )* (*ff)*(*ff) * (0.5*( (*q) + 1.0)));

    /* update                                                               */
    if (Fe <= Fh) {
      if ( e >= h ) Start = h; 
      else End = h;
      f  =  g;  g =  h;  h = e;
      Ff = Fg; Fg = Fh; Fh = Fe;
    } else {
      if ( e <= (h-DBL_EPSILON) ) Start = e; 
      else End = e;
      if ( (Fe <= Fg) || (fabs (g - x)  <= FLT_EPSILON) ) {
        f = g; g = e; 
        Ff = Fg; Fg = Fe;
      } 
      else if ( (Fe <= Ff) || (fabs (f- g) <= FLT_EPSILON) 
                || (fabs (f - h) <= FLT_EPSILON) ) { 
          f = e; Ff = Fe;
      }
    }

  /* end of main loop                                                       */
  }

  /* only reach here if no convergence                                      */
  *tmin = h;     
  *Cmin = Fh;
  return(1);
}

#ifdef HAVE_DISK
/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann
 @version   1.69
 @short     Eclipse checking 
 @param     (double) x coordinate  
 @param     (double) y coordinate  
 @param     (double) R radius  
 @return    (double) angle on circle 
 @heading   Light Curve
 ********************************************************************/
static double toAngle (double x, double y, double R)
{
  double angle;

  x = x / R;

  if (y < 0)
    angle = PI - acos(x);
  else
    angle = PI + acos(x);

  return angle;
}

/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann
 @version   1.70
 @short     Provide disk heights for eclipse testing
 @param     (double*) Height of inner edge of the disk  
 @param     (double*) Height of outer edge of the disk
 @param     (double*) Height of the puffed hot spot
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void diskHeight (double * Height_in, double * Height_out, double * Height_puff)
{
  double  diskExp  = 1.0;             /* Exponent for r dependency of     
                                         scale height                     */ 
  double  diskProp = 1.0;             /* Proportionality factor for disk 
					 height                           */
  double  diskAdd  = 1.0;             /* Additive constant for disk 
					 height                           */

  double  Rout     = Binary[Disk].RCrit * Binary[Disk].Rout;
  double  Rin      = Binary[Disk].RCrit * Binary[Disk].Rin;


  if (Flags.tdisk == 0) 
    {
      diskAdd  = Binary[Disk].Thick;
      diskProp = Binary[Disk].HR;
      diskExp  = 1.0;
    }
  else
    {
      if (Flags.tdisk == 1)   /* isothermal   */ 
	diskExp = 1.5;
      else
	diskExp = (9.0/8.0);  /* reprocessing */ 

      diskAdd  = 0.0;
      diskProp = (Binary[Disk].Thick / pow(Rin, diskExp));
    }

  /* Height of the Disk on the outer side 
   */
  *Height_out        = (diskAdd + diskProp * pow(Rout, diskExp)) / 2.0;

  /* Height of the Disk on the inner side 
   */
  *Height_in         = (diskAdd + diskProp * pow(Rin, diskExp)) / 2.0;

  if (Binary[Disk].puffHS > FLT_EPSILON)
    *Height_puff       = *Height_out + Binary[Disk].puffHS;
  else
    *Height_puff       = *Height_out;

  return;
}

/* This macro yields correct results only if a > 0 and b in [0, 2*PI).
 */
#define DIST_CIRCLE(a, b)  fabs(PI - fabs(PI - fmod((a) - (b), (2.0 * PI) ) ))

int obscuredByPuff(double norm_dist, double height, double height_puff)
{
  if (norm_dist <= 1 && height <= height_puff && height >= (-height_puff))
    {
      double comp = puffHeight(Binary[Disk].puffHS, norm_dist);
      if (height <= comp && height >= (-comp))
	return 1;
    }
  return 0;
}

double a_computed(double x, double y, double b)
{
  double a2;

  if (b > DBL_EPSILON)
    a2 = x*x / ( 1.0 - ( (y*y) / (b*b) ) );
  else
    a2 = x*x;
  
  return (sqrt(a2));
}

		      
/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann
 @version   2.0.3
 @short     Test eclipse of secondary by surrounding disk 
 @param     (double) CosPhase2 Cosine of phase  
 @param     (double) SinPhase2 Sine of phase  
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void SecondaryEclipsedByDisk (int j, double CosPhase2, double SinPhase2,
			      double lzero, double mzero, double nzero)
{
  /* Radii of outer and inner cylinder between which 
   * the disk is situated.
   */
  double   Rout   = Binary[Disk].RCrit * Binary[Disk].Rout;
  double   Rin    = Binary[Disk].RCrit * Binary[Disk].Rin;
  double   R0;

  /* Height of the Disk on the outer side 
   */
  double   HeightIn;
  double   HeightOut;
  double   HeightPuff;
          
  double   x_z;
  double   y_z;
  double   z_z;
  double   x_r, x_r_2;
  double   y_r, y_r_2;
  /* double   z_r, z_r_2; */
  
  static double   Phase0 = 0;
  double   Phase2;
  
  double   A;
  double   B;
  double   A2B;
  double   temp;

  SurfaceElement *SurfPtrS = Surface[Secondary];

  unsigned long  NElemS = Binary[Secondary].NumElem;
  unsigned long  countS = 0;
  
  unsigned long  countS_i  = 0;
  unsigned long  countS_e  = 0;
  unsigned long  countS_e2 = 0;
  unsigned long  countS_v0 = 0;
  unsigned long  countS_v1 = 0;
  unsigned long  countS_v2 = 0;
  unsigned long  countS_v3 = 0;
  unsigned long  countS_v4 = 0;

  double         Tani   = tan(Orbit.Inclination);
  double         Sini   = sin(Orbit.Inclination);
  double         Cosi   = cos(Orbit.Inclination);
  double         a_in,  b_in;
  double         a2, b2;
  double         a_out, b_out;
  double         a_up,  b_up;
  
  double         Zin, Zout;
  double         t_1;
  double         t_2;
  double         t_1_z;
  double         t_2_z;

  double hs_long  = dtor_2pi(Binary[Disk].longitudeHS);     /* longitude of hot spot */
  double hs_ext   = ((Binary[Disk].extentHS / 2.0) * DTOR); /* extent in phi         */

  /* View from above
   */
  if (Tani < FLT_EPSILON)
    return;	  

  if (Flags.elliptic == OFF) {
    /* Phase = (PI/2 + (2*PI/PhaseSteps) * j ); */
    /* shadow of ring moves counter to phase    */
    Phase2 = (PI + PI/2 - (2*PI/PhaseSteps) * j );
  } else {
    if (j == 0)
      Phase0 = Orbit.Phase;
    Phase2 = (PI + Orbit.Phase) - 2 * (Orbit.Phase - Phase0);
  }
  
  SinPhase2 = sin(Phase2); CosPhase2 = cos(Phase2);

  /* diskHeight does not account for projection by cos(inclination) */
  diskHeight (&HeightIn, &HeightOut, &HeightPuff);

  /* the inner ring opening is an ellipse with  */
  /*   a : Rin                                  */
  /*   b = Rin * cos(inclination)               */
  /*   b = projeted on X (max for i = 0         */
  /*   shifted up by HeightIn * Sini            */
  
  b_in = Rin * Cosi;
  a_in = Rin;

  /* the outer ring ellipse                      */
  b_out = Rout * Cosi + HeightOut * Sini;
  a_out = a_computed (Rout, HeightOut * Sini, b_out);

  /* the ellipse for above outer rim             */
  b_up = Rout * Cosi;
  a_up = Rout;
  
  if (Flags.debug[GEO] && Flags.debug[VERBOSE])
    {
      printf("%3d :: %6.3f  %6.3f\n\n", j, HeightIn,  HeightPuff);

      printf("Inner: a = %12.6f  b = %12.6f\n", a_in,  b_in);
      printf("Outer: a = %12.6f  b = %12.6f\n", a_out, b_out);
      printf("Upper: a = %12.6f  b = %12.6f\n", a_up, b_up);
      printf("=============================\n");
    }
  
  while (countS < NElemS)
    {
      /* Invisible anyway
       */
      if (SurfPtrS->visibility == 0)
	{
	  ++countS_i;
	  goto next;
	}

      /* Above disk (and above puff) => visible
       */
      if (SurfPtrS->nu > HeightPuff)
	{
	  ++countS_v0;
	  goto next;
	}

      x_z  = SurfPtrS->lambda;
      y_z  = (-SurfPtrS->mu);
      z_z  = SurfPtrS->nu;

      /* rotation matrix for secondary surface elements 
       */
      x_r =  Cosi*CosPhase2*x_z - Cosi*SinPhase2*y_z + Sini*z_z; 
      y_r =       SinPhase2*x_z +      CosPhase2*y_z;
      /* z_r = -Sini*CosPhase2*x_z + Sini*SinPhase2*y_z + Cosi*z_z; */
      
      /* x_r_2 = x_r  * x_r; */
      y_r_2 = y_r  * y_r;
      /* z_r_2 = z_r  * z_r; */

      if ( (-b_in + HeightIn*Sini) > (-b_up +  HeightOut*Sini) )
	{
	  /* Inner disk hole visible 
	   */
	  double tmp = (x_r - HeightIn*Sini);
	  double x_2;

	  if (tmp > 0) /* visible upper part of star */
	    {
	      ++countS_v1;
	      goto next;
	    }
	  
	  x_2 = tmp * tmp;
	  
	  a2    = a_in * a_in;
	  b2    = b_in * b_in;

	  /* inside 'inner ellipse' (projected disk opening), visible 
	   */
	  if ( (y_r_2/a2 + x_2/b2) < 1)
	    {
	      ++countS_v1;
	      goto next;
	    }

	  x_r_2 = x_r  * x_r;
	  
	  a2    = a_out * a_out;
	  b2    = b_out * b_out;

	  /* inside 'outer ellipse', eclipsed 
	   */
	  if ( (y_r_2/a2 + x_r_2/b2) <= 1)
	    {
	      ++countS_e;
	      goto set_eclipsed;
	    }
	}
      else
	{
	  x_r_2 = x_r  * x_r;

	  a2    = a_out * a_out;
	  b2    = b_out * b_out;
	  
	  /* inside 'outer ellipse', possibly eclipsed 
	   */
	  if ( (y_r_2/a2 + x_r_2/b2) <= 1)
	    {
	      double tmp = (x_r - HeightOut*Sini);
	      double x_2;

	      if (tmp > HeightOut*Sini) /* above disk rim */
		{
		  ++countS_v2;
		  goto next;
		}

	      x_2 = tmp*tmp;
	      
	      a2    = a_up * a_up;
	      b2    = b_up * b_up;
	      
	      if ( (y_r_2/a2 + x_2/b2) <= 1)
		{
		  /* inside upper disk rim ellipse */
		  ++countS_v2;
		  goto next;
		}
	      
	      ++countS_e;
	      goto set_eclipsed;
	    }
	}

      /* outside 'outer ellipse', visible 
       */
      ++countS_v3;
      goto next;

      /*** FIXME ** need to account for the puff */
      
      /*** NOT REACHED **/
      
      /* Compute intersection with inner cylinder
       */
      A    = x_z * CosPhase2 + y_z * SinPhase2;
      R0   = (x_z * x_z) + (y_z * y_z);
      B    = R0 - (Rin * Rin);
      A2B  = (A*A) - B;

      if (A2B <= 0)
	{
	  ++countS_v1;
	  goto next;
	}

      /* z-coordinate of intersection
       */
      temp = sqrt(A2B);

      t_1  = ((-A) - temp) / Tani;
      t_2  = ((-A) + temp) / Tani;

      Zin  = MAX(t_1,t_2) + z_z;

      /* Compute intersection with outer cylinder
       */
      B    = R0 - (Rout * Rout);
      A2B  = (A*A) - B;

      if (A2B <= 0)
	{
	  ++countS_v2;
	  goto next;
	}

      /* Z-coordinate of intersection. Here, t_1, t_2 are
       * t_1 * sin(i), t_2 * sin(i) for the solution
       * of the equation in x-y plane.
       */
      temp = sqrt(A2B);

      t_1  = ((-A) - temp);
      t_2  = ((-A) + temp);

      t_1_z = t_1 / Tani;
      t_2_z = t_2 / Tani;

      Zout = MAX(t_1_z,t_2_z) + z_z;

      if      ( (Zout < (-HeightPuff)) || (Zin > HeightIn && Zout > HeightPuff) )
	{
	  ++countS_v3;
	  goto next;
	}

      if      ((Zin > HeightIn && Zout > HeightOut && Zout <= HeightPuff) ||
	       (Zout < (-HeightOut) && Zout >= (-HeightPuff)))
	{
	  if (Sini > FLT_EPSILON)
	    {
	      double a, hs_dist, t_xy;
	      
	      if (fabs(t_1) < fabs(t_2)) 
		{
		  t_xy = t_1 / Sini;
		}
	      else
		{
		  t_xy = t_2 / Sini;
		} 
		
	      a = toAngle((x_z + t_xy*lzero), (y_z + t_xy*mzero), Rout);

	      hs_dist            = DIST_CIRCLE(a, hs_long);

	      if (hs_dist <= hs_ext)
		{
		  if (obscuredByPuff(fabs(hs_dist/hs_ext), Zout, HeightPuff))
		    {
		      ++countS_e;
		      goto set_eclipsed;
		    }
		}
	    }
	  ++countS_v4;
	  goto next;
	}
      ++countS_e2;

    set_eclipsed:
      SurfPtrS->EclipseFlag = 1;
      SurfPtrS->visibility  = 0;

    next:
      ++SurfPtrS;
      ++countS;
    }

  if (Flags.debug[GEO] && Flags.debug[VERBOSE])
    {
      printf("=======================================\n");
      printf("Phase %3d  SbD: total %6ld  invis %6ld eclipsed %6ld  visible %6ld %6ld %6ld %6ld\n",
	     j, countS, countS_i, countS_e, 
	     countS_v0, countS_v1, countS_v2, countS_v3);
    }
  return;
}

/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann
 @version   1.55
 @short     Test eclipse of disk by itself 
 @param     (double) CosPhase2 Cosine of phase  
 @param     (double) SinPhase2 Sine of phase  
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void DiskEclipsedByDisk (double CosPhase2, double SinPhase2, double lzero, double mzero)
{
  /* Radii of outer and inner cylinder between which 
   * the disk is situated.
   */
  double   Rout   = Binary[Disk].RCrit * Binary[Disk].Rout;
  double   Rin    = Binary[Disk].RCrit * Binary[Disk].Rin;
  double   R0;

  /* Height of the Disk on the outer side 
   */
  double   HeightIn;
  double   HeightOut;
  double   HeightPuff;
          
  double   x_z;
  double   y_z;
  double   z_z;

  double   A;
  double   B;
  double   A2B;

  SurfaceElement *SurfPtr = Surface[Disk];

  unsigned long  NElem = Binary[Disk].NumElem;
  unsigned long  count = 0;

  double         Tani   = tan(Orbit.Inclination);
  double         Sini   = sin(Orbit.Inclination);
  double         Zin, Zout;
  double         t_1;
  double         t_2;
  double         t_1_z;
  double         t_2_z;
  double         temp;

  double hs_long  = dtor_2pi(Binary[Disk].longitudeHS);     /* longitude of hot spot */
  double hs_ext   = ((Binary[Disk].extentHS / 2.0) * DTOR); /* extent in phi         */

  /* View from above
   */
  if (Tani < FLT_EPSILON)
    return;

  diskHeight (&HeightIn, &HeightOut, &HeightPuff);

  while (count < NElem)
    {
      /* Invisible anyway
       */
      if (SurfPtr->visibility == 0)
	goto next;

      /* Above disk => visible
       */
      if (SurfPtr->nu > HeightPuff)
	goto next;

      /* Outer side => never eclipsed by disk
       */
      if (SurfPtr->AreaType == OUT_RECTANGLE)
	goto next;

      x_z  = SurfPtr->lambda;
      y_z  = (-SurfPtr->mu);
      z_z  = SurfPtr->nu;

      A    = x_z * CosPhase2 + y_z * SinPhase2;
      R0   = (x_z * x_z) + (y_z * y_z);

      if (SurfPtr->AreaType == IN_RECTANGLE)
	{
	  /* Compute intersection with inner cylinder
	   */
	  B    = R0 - (Rin * Rin);
	  A2B  = (A*A) - B;
	  
	  if (A2B <= 0)
	    goto outer;
	  
	  /* z-coordinate of intersection
	   */
	  temp = sqrt(A2B);

	  t_1  = ((-A) - temp) / Tani;
	  t_2  = ((-A) + temp) / Tani;
	  
	  Zin  = MAX(t_1,t_2) + z_z;
	  
	  if (Zin < HeightIn && Zin > (-HeightIn))
	    goto set_eclipsed;
	}

    outer:

      /* Compute intersection with outer cylinder
       */
      B    = R0 - (Rout * Rout);
      A2B  = (A*A) - B;

      if (A2B <= 0)
	goto next;

      /* z-coordinate of intersection
       */
      temp = sqrt(A2B);

      t_1  = ((-A) - temp);
      t_2  = ((-A) + temp);

      t_1_z = t_1 / Tani;
      t_2_z = t_2 / Tani;

      Zout = MAX(t_1_z,t_2_z) + z_z;

      if      (Zout <= HeightOut)
	goto set_eclipsed;

      if      (Zout > HeightPuff)
	goto next;

      /* Between HeightOut and HeightPuff, locate
       * intersecting point.
       */
      if (Sini > FLT_EPSILON)
	{
	  double a, hs_dist, t_xy;
	  
	  if (t_1_z > t_2_z) 
	    {
	      t_xy = t_1 / Sini;
	    }
	  else
	    {
	      t_xy = t_2 / Sini;
	    } 
	  
	  a = toAngle((x_z + t_xy*lzero), (y_z + t_xy*mzero), Rout);
	  
	  hs_dist            = DIST_CIRCLE(a, hs_long);
	  
	  if (hs_dist <= hs_ext)
	    {
	      if (obscuredByPuff(fabs(hs_dist/hs_ext), Zout, HeightPuff))
		{
		  goto set_eclipsed;
		}
	    }
	}

      goto next;


    set_eclipsed:
      SurfPtr->EclipseFlag = 1;
      SurfPtr->visibility  = 0;

    next:
      ++SurfPtr;
      ++count;
    }
  
  return;
}



/********************************************************************
 @package   nightfall
 @author    Rainer Wichmann
 @version   1.55
 @short     Test eclipse of primary by disk surrounding secondary 
 @param     (double) CosPhase2 Cosine of phase  
 @param     (double) SinPhase2 Sine of phase  
 @return    (void)
 @heading   Light Curve
 ********************************************************************/
void PrimaryEclipsedByDisk (double CosPhase, double SinPhase, double Phase, 
			    double lzero, double mzero)
{
  /* Radii of outer and inner cylinder between which 
   * the disk is situated.
   */
  double   Rout   = Binary[Disk].RCrit * Binary[Disk].Rout;
  double   Rin    = Binary[Disk].RCrit * Binary[Disk].Rin;

  /* Height of the Disk on the outer side 
   */
  double   HeightIn;
  double   HeightOut;
  double   HeightPuff;
          
  double   x_z;
  double   y_z;
  double   z_z;

  double   A;
  double   B;
  double   A2B;
  double   R0;

  SurfaceElement *SurfPtr = Surface[Primary];

  unsigned long  NElem = Binary[Primary].NumElem;
  unsigned long  count = 0;

  double         Tani   = tan(Orbit.Inclination);
  double         Sini   = sin(Orbit.Inclination);
  double         Zmin;
  double         Zmax;
  double         t_1;
  double         t_2;
  double         t_1_z;
  double         t_2_z;
  double         temp;

  double hs_long  = dtor_2pi(Binary[Disk].longitudeHS);     /* longitude of hot spot */
  double hs_ext   = ((Binary[Disk].extentHS / 2.0) * DTOR); /* extent in phi         */

  /* View from above
   */
  if (Tani < FLT_EPSILON)
    return;

  diskHeight (&HeightIn, &HeightOut, &HeightPuff);

  while (count < NElem)
    {
      /* Invisible anyway
       */
      if (SurfPtr->visibility == 0)
	goto next;

      /* Above disk => visible
       */
      if (SurfPtr->nu > HeightPuff)
	goto next;

      x_z = SurfPtr->lambda;
      y_z = SurfPtr->mu;
      z_z = SurfPtr->nu;

      /* Compute intersection with outer cylinder
       */
      R0  = 1.0 + (x_z * x_z) + (y_z * y_z) - (2.0 * x_z);

      A   = x_z * CosPhase + y_z * SinPhase - CosPhase;
      B   = R0 - (Rout * Rout);
      A2B = (A*A) - B;

      if (A2B <= 0)
	goto next;

      /* z-coordinate of intersection
       */
      temp = sqrt(A2B);
      t_1  = ((-A) - temp);
      t_2  = ((-A) + temp);

      t_1_z = t_1 / Tani;
      t_2_z = t_2 / Tani;

      Zmin  = MIN(t_1_z,t_2_z) + z_z;

      if (Zmin > HeightPuff)
	goto next;

      Zmax  = MAX(t_1_z,t_2_z) + z_z;

      if (Zmax < (-HeightPuff))
	goto next;

      if (HeightPuff != HeightOut)
	{
	  if (Sini > FLT_EPSILON)
	    {
	      double a, hs_dist, t_xy, Z;
	      
	      Z = t_1_z + z_z;

	      if ((Z > HeightOut    && Z <= HeightPuff) ||
		  (Z < (-HeightOut) && Z >= (-HeightPuff)))
		{
		  t_xy = t_1 / Sini;
		  a = toAngle(1.0 - (x_z + t_xy*lzero), -(y_z + t_xy*mzero), Rout);
		  hs_dist            = DIST_CIRCLE(a, hs_long);

		  if (hs_dist <= hs_ext)
		    {
		      if (obscuredByPuff(fabs(hs_dist/hs_ext), Z, HeightPuff))
			{
			  goto set_eclipsed;
			}
		    }
		}

	      Z = t_2_z + z_z;

	      if ((Z > HeightOut    && Z <= HeightPuff) ||
		  (Z < (-HeightOut) && Z >= (-HeightPuff)))
		{
		  t_xy = t_2 / Sini;
		  a = toAngle(1.0 - (x_z + t_xy*lzero), -(y_z + t_xy*mzero), Rout);
		  hs_dist            = DIST_CIRCLE(a, hs_long);

		  if (hs_dist <= hs_ext)
		    {
		      if (obscuredByPuff(fabs(hs_dist/hs_ext), t_2_z + z_z, HeightPuff))
			{
			  goto set_eclipsed;
			}
		    }
		} 
	    }

	  if (Zmin > HeightOut)
	    goto next;
	  
	  if (Zmax < (-HeightOut))
	    goto next;
	}

      /* Compute intersection with inner cylinder
       */
      B   = R0 - (Rin * Rin);
      A2B = (A*A) - B;

      if (A2B > 0)
	{
	  /* z-coordinate of intersection
	   */
	  temp = sqrt(A2B);

	  t_1 = ((-A) - temp) / Tani;
	  t_2 = ((-A) + temp) / Tani;
	  
	  Zmin = MIN(t_1,t_2) + z_z;
	  
	  if (Zmin < (-HeightIn))
	    {
	      Zmax = MAX(t_1,t_2) + z_z;

	      if (Zmax > HeightIn)
		goto next;
	    }
	}

    set_eclipsed:
      SurfPtr->EclipseFlag = 1;
      SurfPtr->visibility  = 0;

    next:
      ++SurfPtr;
      ++count;
    }

  return;
}
#endif
